from __future__ import print_function

import argparse
import glob
import os
import platform
import re
import subprocess
import sys
import tempfile
from collections import defaultdict

def get_suffixes(path):
    return [re.sub(r'^[^_]*_', '_', filename)
             for (dirpath, dirs, files) in os.walk(path)
             for filename in (files)]


def compare_images(before, after, outdir, similar_dir, args):
    before = trim_system_ui("before", before, outdir, args)
    after = trim_system_ui("after", after, outdir, args)
    outname = "comparison_" + os.path.basename(before)
    outpath = outdir + "/" + outname
    subprocess.call(["compare", "-quiet", before, after, outpath])
    result = subprocess.call(["compare", "-quiet", "-fuzz", "3%", "-metric", "AE", before, after, "null:"], stderr=subprocess.STDOUT)
    print("\t", end="")
    if result == 0: # same
        print()
        os.rename(outpath, similar_dir + "/" + outname)
    elif result == 1: # different
        print()
    else:
        print("error")

    return result


def trim_system_ui(prefix, imagefile, outdir, args):
    if "_fullScreen" in imagefile:
        return imagefile
    outpath = imagefile

    if args.osversion == "10.6":
        titlebarHeight = 22 * args.dppx
        chop = "0x%d" % titlebarHeight
        outpath = outdir + "/chop_" + prefix + "_" + os.path.basename(imagefile)
        subprocess.call(["convert", imagefile, "-chop", chop, outpath])

    return outpath


def compare_dirs(before, after, outdir, args):
    similar_dir = outdir + "/similar"
    os.mkdir(similar_dir)
    sorted_suffixes = sorted(set(get_suffixes(before) + get_suffixes(after)))
    maxFWidth = reduce(lambda x, y: max(x, len(y)), sorted_suffixes, 0)

    print("SCREENSHOT SUFFIX".ljust(maxFWidth), "DIFFERING PIXELS (WITH FUZZ)")
    resultDict = defaultdict(list)
    for f in sorted_suffixes:
        image1 = glob.glob(sys.argv[1] + "/*" + f)
        image2 = glob.glob(sys.argv[2] + "/*" + f)
        if not image1:
            print("{0} exists in after but not in before".format(f))
            continue
        if not image2:
            print("{0} exists in before but not in after".format(f))
            continue
        print(f, "", end="".ljust(maxFWidth - len(f)))
        sys.stdout.flush()
        result = compare_images(image1[0], image2[0], outdir, similar_dir, args)
        resultDict[result].append(f)
    print("{0} similar, {1} different, {2} errors".format(len(resultDict[0]), len(resultDict[1]), len(resultDict[2])))

def cli(args=sys.argv[1:]):
    parser = argparse.ArgumentParser(description='Compare screenshot files or directories for differences')
    parser.add_argument("before", help="Image file or directory of images")
    parser.add_argument("after", help="Image file or directory of images")
    parser.add_argument("--osversion", choices=["10.6"],
                        help="Operating system the images are from so that OS UI"+
                        " can be trimmed off. Not necessary for modern OS X.")
    parser.add_argument("--dppx", type=float, default=1.0, help="Scale factor to use for cropping system UI")

    args = parser.parse_args()

    before = args.before
    after = args.after
    outdir = tempfile.mkdtemp()
    print("Image comparison results:", outdir)
    print()

    if (os.path.isdir(before) and os.path.isdir(after)):
        compare_dirs(before, after, outdir, args)
    elif (os.path.isfile(before) and os.path.isfile(after)):
        compare_images(before, after, outdir, outdir, args)
    else:
        print("Two files or two directories expected")
        return

if __name__ == "__main__":
    cli()



#!/usr/bin/env python

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.

import optparse
import os
import subprocess
import sys

from mozrunner import CLI as MozRunnerCLI, runners


class CLI(MozRunnerCLI):
    """Command line interface."""

    module = "mozscreenshots"

    def __init__(self, args=sys.argv[1:]):
        """mozscreenshots command line entry point"""

        self.metadata = None
        if self.module in sys.modules:
            self.metadata = getattr(sys.modules[self.module],
                                    'package_metadata',
                                    {})
            version = self.metadata.get('Version')
            parser_args = {'description': self.metadata.get('Summary')}
            if version:
                parser_args['version'] = "%prog " + version

        self.parser = optparse.OptionParser()
        self.add_options(self.parser)
        (self.options, self.args) = self.parser.parse_args(args)

        if (self.options.list_sets):
            subprocess.call(["ls " + os.path.dirname(os.path.abspath(__file__)) + "/extension/configurations/*.jsm | sed -e 's/.*\///' | sed -e 's/\.jsm//' | xargs"], shell=True)
            sys.exit(0)
            return

        os.environ["MOZSCREENSHOTS_SETS"] = ",".join(self.args)
        if self.options.interactive:
            os.environ["MOZSCREENSHOTS_INTERACTIVE"] = "1"

        # choose appropriate runner and profile classes
        try:
            self.runner_class = runners[self.options.app]
        except KeyError:
            self.parser.error('Application "%s" unknown (should be one of "firefox" or "thunderbird")' % self.options.app)

    def add_options(self, parser):
        # add profile options
        MozRunnerCLI.add_options(self, parser)

        parser.add_option('--list-sets', dest='list_sets',
                          action='store_true',
                          help="List the available sets and exit.")

    def run(self):
        module_path = os.path.dirname(os.path.abspath(__file__))
        # append the mozscreenshots extension
        self.options.addons.append(module_path + '/extension/')

        # run in the foreground so we don't get screenshots of the console
        self.options.appArgs.append("--foreground")

        self.options.prefs_files.append(module_path + '/prefs.json')

        super(CLI, self).run()


def cli(args=sys.argv[1:]):
    CLI(args).run()

if __name__ == "__main__":
    cli()



import argparse


// mozci-trigger --rev 5f6ca9194dd9 -b "Rev4 MacOSX Snow Leopard 10.6 try opt test mochitest-browser-chrome-2" --file https://ftp.mozilla.org/pub/mozilla.org/firefox/try-builds/mozilla@noorenberghe.ca-5f6ca9194dd9/try-macosx64/firefox-41.0a1.en-US.mac.dmg --file https://people.mozilla.org/~mnoorenberghe/tmp/screenshots_mbc2.tests.zip

platforms = {
    "10.6": {
        
    }
}

parser = argparse.ArgumentParser(description="Run mozscreenshots on Try server")
parser.add_argument("--10.6", action="store_true")
args = parser.parse_args()
print dir(args)
print getattr(args, "10.6")







import argparse
import json
import os
import os.path
import requests

def fetch_screenshots(rev, project):
    resultset_url = 'https://treeherder.mozilla.org/api/project/%s/resultset/?count=1&full=true&revision=%s' % (project, rev)
    resultset = requests.get(resultset_url).json()

    if resultset['meta']['count'] == 0:
        print 'No results for that revision'
        return

    result_set_id = resultset['results'][0]['id']
    print "Result Set ID: %d" % result_set_id

    jobs_url = 'https://treeherder.mozilla.org/api/project/%s/jobs/?count=2000&result_set_id=%d' % (project, result_set_id)
    jobs = requests.get(jobs_url).json()

    for job in jobs['results']:
        if job['job_type_name'] != 'Mochitest Browser Chrome':
            continue
        job_id = job['id']
        fetch_artifacts_for_job(job_id, job['platform'], rev, project)


def fetch_artifacts_for_job(job_id, platform, rev, project):
    print "Fetching artifacts for job id: %d" % job_id
    artifacts_url = 'https://treeherder.mozilla.org/api/project/%s/artifact/?job_id=%d&name=Job+Info&type=json' % (project, job_id)
    artifacts = requests.get(artifacts_url).json()

    for artifact in artifacts:
        if "blob" not in artifact:
            continue

        blob = artifact["blob"]

        if "job_details" not in blob:
            continue

        job_details = blob["job_details"]

        for detail in job_details:
            if not detail["value"].endswith(".png"):
                continue
            try:
                os.mkdir("%s-%s-%d" % (rev, platform, job_id))
            except OSError:
                pass

            download_screenshot(detail["url"], "%s-%s-%d/%s" % (rev, platform, job_id, detail["value"]))


def download_screenshot(url, filepath):
    print "Downloading %s" % filepath,
    if os.path.isfile(filepath):
        print "- Not overwriting existing file"
        return
    else:
        print
    image = requests.get(url)
    file = open(filepath, 'wb')
    file.write(image.content)
    file.close()

def cli():
    parser = argparse.ArgumentParser(description='Fetch screenshots from automation')
    parser.add_argument('-r', '--rev', required=True,
                        help="Revision to fetch screenshots from")
    parser.add_argument('--project', default="try",
                        help="Project that the revision is from. [Default=try]")
    args = parser.parse_args()
    fetch_screenshots(args.rev, args.project)


if __name__ == "__main__":
    cli()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.

from setuptools import setup

PACKAGE_NAME = 'mozscreenshots'
PACKAGE_VERSION = '0.2'

desc = """Takes screenshots of different states of a Mozilla application"""

deps = [
    'mozrunner >= 5.0',
    'requests == 2.7.0',
]

setup(name=PACKAGE_NAME,
      version=PACKAGE_VERSION,
      description=desc,
      long_description="",
      classifiers=['Environment :: Console',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
                   'Natural Language :: English',
                   'Operating System :: OS Independent',
                   'Programming Language :: JavaScript',
                   'Programming Language :: Python',
                   'Topic :: Software Development :: Libraries :: Python Modules',
                   ],
      keywords=['mozilla', 'screenshots', 'Firefox'],
      author='Matthew Noorenberghe',
      author_email='MattN+mozscreenshots@mozilla.com',
      url='https://github.com/mnoorenberghe/mozscreenshots',
      download_url='https://github.com/mnoorenberghe/mozscreenshots/tarball/' + PACKAGE_VERSION,
      license='MPL 2.0',
      packages=['mozscreenshots'],
      package_dir={'mozscreenshots': 'mozscreenshots'},
      package_data={'mozscreenshots': [
            'extension/*.*',
            'extension/configurations/*.jsm',
            'extension/lib/*.*',
            'prefs.json'
            ]},
      zip_safe=False,
      install_requires = deps,
      entry_points="""
# -*- Entry points: -*-
[console_scripts]
mozscreenshots = mozscreenshots:cli
fetch_screenshots = mozscreenshots.fetch_screenshots:cli
compare_screenshots = mozscreenshots.compare_screenshots:cli
""",
    )
